@@ -18,9 +18,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.141 2008/02/20 09:39:41 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.143 2008/05/22 13:20:34 lav Exp $ */
 
 #include <config.h>
+#include <stddef.h>
 #include <errno.h>
 #include <unistd.h>
 #include <ctype.h>
@@ -1066,6 +1067,10 @@ char *ArgV::CombineQuoted(int start) const
       res.append(' ');
    }
 }
+char *ArgV::CombineCmd(int i) const
+{
+   return i>=count()-1 ? Combine(i) : CombineQuoted(i);
+}
 
 const char *CmdExec::GetFullCommandName(const char *cmd)
 {
