@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.57 2006/08/22 06:47:25 lav Exp $ */
+/* $Id: CmdExec.h,v 1.66 2007/09/17 14:01:27 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -45,7 +45,7 @@ typedef Job * (*cmd_creator_t)(class CmdExec *parent);
 class CmdFeeder
 {
 public:
-   char *saved_buf;
+   xstring_c saved_buf;
    CmdFeeder *prev;
    virtual const char *NextCmd(class CmdExec *exec,const char *prompt) = 0;
    virtual ~CmdFeeder() {}
@@ -63,21 +63,22 @@ class CmdExec : public SessionJob
 {
 public:
 // current command data
-   char *cmd;
-   ArgV *args;
-   FDStream *output;
+   Ref<ArgV> args;
+   Ref<FDStream> output;
    bool background;
    int	 exit_code;
+   int	 prev_exit_code;
 
 private:
-   char *next_cmd;
-   char *cmd_buf;
+   Buffer cmd_buf;
    bool partial_cmd;
    int alias_field; // length of expanded alias (and ttl for used_aliases)
 
    TouchedAlias *used_aliases;
    void free_used_aliases();
 
+   void skip_cmd(int len);
+
    enum
    {
       COND_ANY,
@@ -94,8 +95,7 @@ private:
       const char  *long_desc;
    };
    static const cmd_rec static_cmd_table[];
-   static cmd_rec *dyn_cmd_table;
-   static int dyn_cmd_table_count;
+   static xarray<cmd_rec> dyn_cmd_table;
 
    static int find_cmd(const char *cmd_name,const cmd_rec **ret);
 
@@ -112,8 +112,6 @@ private:
    CmdFeeder *feeder;
    bool feeder_called;
 
-   int	 prev_exit_code;
-
    enum builtins
    {
       BUILTIN_NONE=0,
@@ -125,11 +123,11 @@ private:
       builtin;
 
    FileAccess::Path old_cwd;
-   char *old_lcwd;
-   char *slot;
+   xstring_c old_lcwd;
+   xstring_c slot;
 
-   GlobURL *glob;
-   ArgV *args_glob;
+   Ref<GlobURL> glob;
+   Ref<ArgV> args_glob;
 
    int redirections;
 
@@ -141,7 +139,7 @@ private:
    bool SameQueueParameters(CmdExec *,const char *);
    int max_waiting;
 
-   FileAccess *saved_session;
+   FileAccessRef saved_session;
    void ReuseSavedSession();
    void RevertToSavedSession();
 
@@ -150,8 +148,8 @@ public:
    void FeedArgV(const ArgV *,int start=0);
    void PrependCmd(const char *c);
    void ExecParsed(ArgV *a,FDStream *o=0,bool b=false);
-   static void unquote(char *buf,const char *str);
-   static char *unquote(const char *str);
+   static int unquote(char *buf,const char *str);  // returns buf length
+   static const char *unquote(const char *str);
    static bool needs_quotation(const char *buf);
    static bool quotable(char c,char in_quotes);
    static bool is_space(char c) { return c==' ' || c=='\t'; }
@@ -173,16 +171,16 @@ public:
    int ExitCode() { return exit_code; }
    int Do();
    void PrintStatus(int,const char *prefix="\t");
-   void ShowRunStatus(StatusLine *s);
+   void ShowRunStatus(const SMTaskRef<StatusLine>& s);
    int AcceptSig(int sig);
 
-   char *FormatPrompt(const char *scan);
-   char *MakePrompt();
+   const char *FormatPrompt(const char *scan);
+   const char *MakePrompt();
 
    bool interactive;
    bool top_level;
    bool verbose;
-   StatusLine *status_line;
+   SMTaskRef<StatusLine> status_line;
    void SetCmdFeeder(CmdFeeder *new_feeder);
    void	RemoveFeeder();
 
@@ -216,11 +214,7 @@ public:
 	 top_level=true;
 	 Reconfig(0);
       }
-   void SetStatusLine(StatusLine *s)
-      {
-	 Delete(status_line);
-	 status_line=s;
-      }
+   void SetStatusLine(StatusLine *s) { status_line=s; }
 
    static void RegisterCommand(const char *name,cmd_creator_t creator,
       const char *short_name=0,const char *long_name=0);
@@ -242,7 +236,7 @@ public:
    void print_cmd_help(const char *cmd);
    void print_cmd_index();
 
-   static const cmd_rec *CmdByIndex(int i);
+   static const char *CmdByIndex(int i);
 
    int	 last_bg;
    bool	 wait_all;
@@ -251,7 +245,7 @@ public:
 
    void ChangeSlot(const char *n);
 
-   static CmdExec *top;
+   static JobRef<CmdExec> top;
 };
 
 extern const char * const bookmark_subcmd[];
