@@ -18,11 +18,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: DummyProto.cc,v 1.7 2006/06/13 14:35:34 lav Exp $ */
+/* $Id: DummyProto.cc,v 1.10 2008/07/23 08:46:09 lav Exp $ */
 
 #include <config.h>
 
 #include "DummyProto.h"
+#include <stddef.h>
 
 int DummyProto::Do() { return STALL; }
 int DummyProto::Done() { return NO_HOST; }
@@ -34,10 +35,9 @@ int DummyProto::StoreStatus() { return NO_HOST; }
 
 class DummyDirList : public DirList
 {
-   FA *p;
 public:
-   DummyDirList(FA *p1,ArgV *a) : DirList(a) { p=p1; }
-   int Do() { SetError(p->StrError(FA::NO_HOST)); return STALL; }
+   DummyDirList(FA *p1,ArgV *a) : DirList(p1,a) {}
+   int Do() { SetError(session->StrError(FA::NO_HOST)); return STALL; }
    const char *Status() { return ""; }
 };
 class DummyListInfo : public ListInfo
@@ -56,20 +56,11 @@ ListInfo *DummyProto::MakeListInfo(const char *path)
    return new DummyListInfo(this);
 }
 
-
-DummyNoProto::DummyNoProto(const char *p)
-{
-   proto=xstrdup(p);
-}
-DummyNoProto::~DummyNoProto()
-{
-   xfree(proto);
-}
 const char *DummyNoProto::GetProto() const { return proto; }
 FileAccess *DummyNoProto::Clone() const { return new DummyNoProto(proto); }
 const char *DummyNoProto::StrError(int)
 {
-   static char str[128];
-   sprintf(str,"%.32s%s",proto,_(" - not supported protocol"));
+   static xstring str;
+   str.vset(proto.get(),_(" - not supported protocol"),NULL);
    return str;
 }
