@@ -18,24 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.h,v 1.106 2008/02/17 08:19:23 lav Exp $ */
+/* $Id: FileAccess.h,v 1.109 2008/08/05 12:10:08 lav Exp $ */
 
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
 
-#include <time.h>
 #include <sys/types.h>
-
-#ifdef HAVE_SYS_STROPTS_H
-# include <sys/stropts.h>
-#endif
-
-#ifdef HAVE_SYS_POLL_H
-# include <sys/poll.h>
-#else
-# include <poll.h>
-#endif
-#include <assert.h>
+#include <time.h>
 
 #include "SMTask.h"
 #include "trio.h"
@@ -101,6 +90,7 @@ public:
       Path(const char *new_path) { init(); Set(new_path); }
       Path(const char *new_path,bool new_is_file,const char *new_url=0,int new_device_prefix_len=0)
 	 { init(); Set(new_path,new_is_file,new_url,new_device_prefix_len); }
+      ~Path();
       void Set(const Path*);
       void Set(const Path &o) { Set(&o); }
       void Set(const char *new_path,bool new_is_file=false,const char *new_url=0,int device_prefix_len=0);
@@ -167,11 +157,7 @@ protected:
       {
 	 real_cwd.set(c);
       }
-   void set_home(const char *h)
-      {
-	 home.Set(h);
-	 ExpandTildeInCWD();
-      }
+   void set_home(const char *h);
 
    off_t  entity_size; // size of file to be sent
    time_t entity_date; // date of file to be sent
@@ -423,7 +409,7 @@ public:
    FileAccessRef() {}
    FileAccessRef(FileAccess *p) : SMTaskRef<FileAccess>(p) {}
    ~FileAccessRef() { reuse(); }
-   const FileAccessRef& operator=(FileAccess *p) { reuse(); ptr=SMTask::MakeRef(p); return *this; }
+   const FileAccessRef& operator=(FileAccess *p);
 
    template<class T> const SMTaskRef<T>& Cast() const
       { void(static_cast<T*>(this->ptr)); return *(const SMTaskRef<T>*)this; }
@@ -500,6 +486,8 @@ protected:
    unsigned need;
    bool follow_symlinks;
 
+   void PrepareToDie();
+
 public:
    ListInfo(FileAccess *session,const char *path);
    ~ListInfo();
