@@ -18,11 +18,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.52 2006/08/29 07:25:26 lav Exp $ */
+/* $Id: FileSet.cc,v 1.65 2008/05/22 14:50:30 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
 
+#include <stddef.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <utime.h>
@@ -42,10 +43,11 @@
 #include "IdNameCache.h"
 #include "PatternSet.h"
 
-#define NO_SIZE	     (-1L)
-#define NO_SIZE_YET  (-2L)
-#define NO_DATE	     ((time_t)-1L)
-#define NO_DATE_YET  ((time_t)-2L)
+CDECL_BEGIN
+#include "lstat.h"
+CDECL_END
+
+#define fnum files.count()
 
 void  FileInfo::Merge(const FileInfo& f)
 {
@@ -55,8 +57,8 @@ void  FileInfo::Merge(const FileInfo& f)
    int dif=(~defined)&f.defined;
    if(dif&MODE)
       SetMode(f.mode);
-   if(dif&DATE || (defined&DATE && f.defined&DATE && f.date_prec<date_prec))
-      SetDate(f.date,f.date_prec);
+   if(dif&DATE || (defined&DATE && f.defined&DATE && f.date.ts_prec<date.ts_prec))
+      SetDate(f.date,f.date.ts_prec);
    if(dif&TYPE)
       SetType(f.filetype);
    if(dif&SYMLINK)
@@ -69,17 +71,6 @@ void  FileInfo::Merge(const FileInfo& f)
       SetNlink(f.nlinks);
 }
 
-void FileInfo::SetName(const char *n)
-{
-   if(n==name)
-      return;
-   // in case of n being tail of name, dup it first
-   char *n1=xstrdup(n);
-   xfree(name);
-   name=n1;
-   defined|=NAME;
-}
-
 void FileInfo::SetUser(const char *u)
 {
    if(u==user)
@@ -98,19 +89,12 @@ void FileInfo::SetGroup(const char *g)
 
 void FileSet::add_before(int pos,FileInfo *fi)
 {
-   if(fnum==allocated)
-   {
-      allocated=allocated?allocated*2:16;
-      files=files_sort=(FileInfo**)xrealloc(files,allocated*sizeof(*files));
-   }
-   memmove(files+pos+1, files+pos, sizeof(*files)*(fnum-pos));
-   files[pos]=fi;
-   fnum++;
+   files.insert(fi,pos);
 }
 void FileSet::Add(FileInfo *fi)
 {
    assert(!sorted);
-   if(!(fi->defined & fi->NAME))
+   if(!fi->name)
    {
       delete fi;
       return;
@@ -128,10 +112,7 @@ void FileSet::Add(FileInfo *fi)
 void FileSet::Sub(int i)
 {
    assert(!sorted);
-   if(i>=fnum)
-      abort();
-   delete files[i];
-   memmove(files+i,files+i+1,(--fnum-i)*sizeof(*files));
+   files.remove(i);
    if(ind>i)
       ind--;
 }
@@ -140,7 +121,7 @@ void FileSet::Merge(const FileSet *set)
 {
    for(int i=0; i<set->fnum; i++)
    {
-      FileInfo *fi=set->files[i];
+      const Ref<FileInfo>& fi=set->files[i];
       int pos = FindGEIndByName(fi->name);
       if(pos < fnum && !strcmp(files[pos]->name,fi->name))
 	 files[pos]->Merge(*fi);
@@ -155,35 +136,39 @@ void FileSet::PrependPath(const char *path)
       files[i]->SetName(dir_file(path, files[i]->name));
 }
 
+FileSet::FileSet()
+   : ind(0)
+{
+}
+
+FileSet::~FileSet()
+{
+}
+
 /* we don't copy the sort state--nothing needs it, and it'd
  * be a bit of a pain to implement. */
 FileSet::FileSet(FileSet const *set)
 {
    ind=set->ind;
-   fnum=set->fnum;
-   sorted=false;
-   if(fnum==0)
-      files=files_sort=0;
-   else
-      files=files_sort=(FileInfo**)xmalloc((allocated=fnum)*sizeof(*files));
-   for(int i=0; i<fnum; i++)
-      files[i]=new FileInfo(*(set->files[i]));
+   for(int i=0; i<set->fnum; i++)
+      files.append(new FileInfo(*(set->files[i])));
 }
 
 static int (*compare)(const char *s1, const char *s2);
 static int rev_cmp;
+static RefArray<FileInfo> *files_cmp;
 
 static int sort_name(const void *s1, const void *s2)
 {
-   const FileInfo *p1 = *(const FileInfo **) s1;
-   const FileInfo *p2 = *(const FileInfo **) s2;
+   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
+   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
    return compare(p1->name, p2->name) * rev_cmp;
 }
 
 static int sort_size(const void *s1, const void *s2)
 {
-   const FileInfo *p1 = *(const FileInfo **) s1;
-   const FileInfo *p2 = *(const FileInfo **) s2;
+   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
+   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
    if(p1->size > p2->size) return -rev_cmp;
    if(p1->size < p2->size) return rev_cmp;
    return 0;
@@ -191,8 +176,8 @@ static int sort_size(const void *s1, const void *s2)
 
 static int sort_dirs(const void *s1, const void *s2)
 {
-   const FileInfo *p1 = *(const FileInfo **) s1;
-   const FileInfo *p2 = *(const FileInfo **) s2;
+   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
+   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
    if(p1->filetype == FileInfo::DIRECTORY && !p2->filetype == FileInfo::DIRECTORY) return -rev_cmp;
    if(!p1->filetype == FileInfo::DIRECTORY && p2->filetype == FileInfo::DIRECTORY) return rev_cmp;
    return 0;
@@ -200,8 +185,8 @@ static int sort_dirs(const void *s1, const void *s2)
 
 static int sort_rank(const void *s1, const void *s2)
 {
-   const FileInfo *p1 = *(const FileInfo **) s1;
-   const FileInfo *p2 = *(const FileInfo **) s2;
+   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
+   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
    if(p1->GetRank()==p2->GetRank())
       return sort_name(s1,s2);
    return p1->GetRank()<p2->GetRank() ? -rev_cmp : rev_cmp;
@@ -209,16 +194,13 @@ static int sort_rank(const void *s1, const void *s2)
 
 static int sort_date(const void *s1, const void *s2)
 {
-   const FileInfo *p1 = *(const FileInfo **) s1;
-   const FileInfo *p2 = *(const FileInfo **) s2;
+   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
+   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
    if(p1->date==p2->date)
       return sort_name(s1,s2);
    return p1->date>p2->date ? -rev_cmp : rev_cmp;
 }
 
-/* files_sort is an alias of files when sort == NAME (since
- * files is always sorted by name), and an independant array
- * of pointers (pointing to the same data) otherwise. */
 void FileSet::Sort(sort_e newsort, bool casefold, bool reverse)
 {
    if(newsort == BYNAME && !casefold && !reverse) {
@@ -226,25 +208,24 @@ void FileSet::Sort(sort_e newsort, bool casefold, bool reverse)
       return;
    }
 
-   if(files_sort == files) {
-      files_sort=(FileInfo**)xmalloc(fnum*sizeof(FileInfo *));
-      for(int i=0; i < fnum; i++)
-	 files_sort[i] = files[i];
-   }
-
-   sorted=true;
-
    if(casefold) compare = strcasecmp;
    else compare = strcmp;
 
    rev_cmp=(reverse?-1:1);
+   files_cmp=&files;
 
+   sorted.truncate();
+   for(int i=0; i<fnum; i++)
+      sorted.append(i);
+
+   int *s=sorted.get_non_const();
+   int sz=sorted.get_element_size();
    switch(newsort) {
-   case BYNAME: qsort(files_sort, fnum, sizeof(FileInfo *), sort_name); break;
-   case BYSIZE: qsort(files_sort, fnum, sizeof(FileInfo *), sort_size); break;
-   case DIRSFIRST: qsort(files_sort, fnum, sizeof(FileInfo *), sort_dirs); break;
-   case BYRANK: qsort(files_sort, fnum, sizeof(FileInfo *), sort_rank); break;
-   case BYDATE: qsort(files_sort, fnum, sizeof(FileInfo *), sort_date); break;
+   case BYNAME: qsort(s, fnum, sz, sort_name); break;
+   case BYSIZE: qsort(s, fnum, sz, sort_size); break;
+   case DIRSFIRST: qsort(s, fnum, sz, sort_dirs); break;
+   case BYRANK: qsort(s, fnum, sz, sort_rank); break;
+   case BYDATE: qsort(s, fnum, sz, sort_date); break;
    }
 }
 
@@ -252,29 +233,16 @@ void FileSet::Sort(sort_e newsort, bool casefold, bool reverse)
  * (Nothing uses this ... */
 void FileSet::Unsort()
 {
-   if(!sorted) return;
-   xfree(files_sort);
-   files_sort=files;
-   sorted=false;
+   sorted.unset();
 }
 
 void FileSet::Empty()
 {
    Unsort();
-   for(int i=0; i<fnum; i++)
-      delete files[i];
-   xfree(files);
-   files=0; files_sort=0;
-   fnum=0;
-   allocated=0;
+   files.unset();
    ind=0;
 }
 
-FileSet::~FileSet()
-{
-   Empty();
-}
-
 void FileSet::SubtractSame(const FileSet *set,int ignore)
 {
    for(int i=0; i<fnum; i++)
@@ -309,34 +277,21 @@ void FileSet::SubtractSameType(const FileSet *set)
    }
 }
 
-void FileSet::SubtractOlderThan(time_t t)
+void FileSet::SubtractTimeCmp(bool (FileInfo::*cmp)(time_t) const,time_t t)
 {
    for(int i=0; i<fnum; i++)
    {
       if(files[i]->defined&FileInfo::TYPE
       && files[i]->filetype!=FileInfo::NORMAL)
 	 continue;
-      if(files[i]->OlderThan(t))
-      {
-	 Sub(i);
-	 i--;
-      }
-   }
-}
-void FileSet::SubtractNewerThan(time_t t)
-{
-   for(int i=0; i<fnum; i++)
-   {
-      if(files[i]->defined&FileInfo::TYPE
-      && files[i]->filetype!=FileInfo::NORMAL)
-	 continue;
-      if(files[i]->NewerThan(t))
+      if((files[i].get()->*cmp)(t))
       {
 	 Sub(i);
 	 i--;
       }
    }
 }
+
 void FileSet::SubtractSizeOutside(const Range *r)
 {
    for(int i=0; i<fnum; i++)
@@ -403,7 +358,7 @@ void FileSet::ExcludeUnaccessible()
    }
 }
 
-bool  FileInfo::SameAs(const FileInfo *fi,int ignore)
+bool  FileInfo::SameAs(const FileInfo *fi,int ignore) const
 {
    if(defined&NAME && fi->defined&NAME)
       if(strcmp(name,fi->name))
@@ -421,9 +376,9 @@ bool  FileInfo::SameAs(const FileInfo *fi,int ignore)
 
    if(defined&DATE && fi->defined&DATE && !(ignore&DATE))
    {
-      time_t p=date_prec;
-      if(p<fi->date_prec)
-	 p=fi->date_prec;
+      time_t p=date.ts_prec;
+      if(p<fi->date.ts_prec)
+	 p=fi->date.ts_prec;
       if(!(ignore&IGNORE_DATE_IF_OLDER && date<fi->date)
       && labs(date-fi->date)>p)
 	 return false;
@@ -440,20 +395,28 @@ bool  FileInfo::SameAs(const FileInfo *fi,int ignore)
    return true;
 }
 
-bool  FileInfo::OlderThan(time_t t)
-{
-   return((defined&DATE) && date<t);
-}
-bool  FileInfo::NewerThan(time_t t)
+bool  FileInfo::NotOlderThan(time_t t) const
 {
    return((defined&DATE) && date>=t);
 }
-bool  FileInfo::SizeOutside(const Range *r)
+bool  FileInfo::NotNewerThan(time_t t) const
+{
+   return((defined&DATE) && date<=t);
+}
+bool  FileInfo::OlderThan(time_t t) const
+{
+   return((defined&DATE) && date<t);
+}
+bool  FileInfo::NewerThan(time_t t) const
+{
+   return((defined&DATE) && date>t);
+}
+bool  FileInfo::SizeOutside(const Range *r) const
 {
    return((defined&SIZE) && !r->Match(size));
 }
 
-void FileSet::Count(int *d,int *f,int *s,int *o)
+void FileSet::Count(int *d,int *f,int *s,int *o) const
 {
    for(int i=0; i<fnum; i++)
    {
@@ -505,12 +468,12 @@ FileInfo *FileSet::FindByName(const char *name) const
    int n = FindGEIndByName(name);
 
    if(n < fnum && !strcmp(files[n]->name,name))
-      return files[n];
+      return files[n].get_non_const();
 
    return 0;
 }
 
-static bool do_exclude_match(const char *prefix,FileInfo *fi,PatternSet *x)
+static bool do_exclude_match(const char *prefix,const FileInfo *fi,const PatternSet *x)
 {
    const char *name=dir_file(prefix,fi->name);
    if(fi->defined&fi->TYPE && fi->filetype==fi->DIRECTORY)
@@ -522,7 +485,7 @@ static bool do_exclude_match(const char *prefix,FileInfo *fi,PatternSet *x)
    return x->MatchExclude(name);
 }
 
-void  FileSet::Exclude(const char *prefix,PatternSet *x)
+void  FileSet::Exclude(const char *prefix,const PatternSet *x)
 {
    if(!x)
       return;
@@ -536,44 +499,23 @@ void  FileSet::Exclude(const char *prefix,PatternSet *x)
    }
 }
 
-
-// *** Manipulations with set of local files
-
 #if 0
-void FileSet::LocalRemove(const char *dir)
+void FileSet::Dump(const char *tag) const
 {
-   FileInfo *file;
+   printf("%s:",tag);
    for(int i=0; i<fnum; i++)
-   {
-      file=files[i];
-      if(file->defined & file->DATE)
-      {
-	 const char *local_name=dir_file(dir,file->name);
-
-	 if(!(file->defined & file->TYPE)
-	 || file->filetype==file->DIRECTORY)
-	 {
-	    int res=rmdir(local_name);
-	    if(res==0)
-	       continue;
-	    res=remove(local_name);
-	    if(res==0)
-	       continue;
-	    truncate_file_tree(local_name);
-	    continue;
-	 }
-	 remove(local_name);
-      }
-   }
+      printf(" %s",files[i]->name.get());
+   printf("\n");
 }
 #endif
 
+// *** Manipulations with set of local files
+
 void FileSet::LocalUtime(const char *dir,bool only_dirs)
 {
-   FileInfo *file;
    for(int i=0; i<fnum; i++)
    {
-      file=files[i];
+      const Ref<FileInfo>& file=files[i];
       if(file->defined & file->DATE)
       {
 	 if(!(file->defined & file->TYPE))
@@ -588,17 +530,16 @@ void FileSet::LocalUtime(const char *dir,bool only_dirs)
 	 struct stat st;
 	 ut.actime=ut.modtime=file->date;
 
-	 if(stat(local_name,&st)!=-1 && st.st_mtime!=file->date)
+	 if(stat(local_name,&st)!=-1 && labs(st.st_mtime-file->date)>file->date.ts_prec)
 	    utime(local_name,&ut);
       }
    }
 }
 void FileSet::LocalChmod(const char *dir,mode_t mask)
 {
-   FileInfo *file;
    for(int i=0; i<fnum; i++)
    {
-      file=files[i];
+      const Ref<FileInfo>& file=files[i];
       if(file->defined & file->MODE)
       {
 	 if(file->defined & file->TYPE
@@ -617,10 +558,9 @@ void FileSet::LocalChmod(const char *dir,mode_t mask)
 }
 void FileSet::LocalChown(const char *dir)
 {
-   FileInfo *file;
    for(int i=0; i<fnum; i++)
    {
-      file=files[i];
+      const Ref<FileInfo>& file=files[i];
       if(file->defined & (file->USER|file->GROUP))
       {
 #ifndef HAVE_LCHOWN
@@ -663,7 +603,9 @@ FileInfo * FileSet::operator[](int i) const
 {
    if(i>=fnum || i<0)
       return 0;
-   return files_sort[i];
+   if(sorted)
+      i=sorted[i];
+   return files[i].get_non_const();
 }
 
 FileInfo *FileSet::curr()
@@ -689,33 +631,36 @@ void FileInfo::Init()
    filetype=UNKNOWN;
    mode=(mode_t)-1;
    date=NO_DATE;
-   date_prec=0;
    size=NO_SIZE;
    nlinks=0;
-   name=0;
    defined=0;
-   symlink=0;
-   data=0;
    user=0; group=0;
    rank=0;
-   longname=0;
 }
 
 FileInfo::FileInfo(const FileInfo &fi)
 {
    Init();
-   name=xstrdup(fi.name);
-   symlink=xstrdup(fi.symlink);
+   name.set(fi.name);
+   symlink.set(fi.symlink);
    user=fi.user;
    group=fi.group;
    defined=fi.defined;
    filetype=fi.filetype;
    mode=fi.mode;
    date=fi.date;
-   date_prec=fi.date_prec;
    size=fi.size;
    nlinks=fi.nlinks;
-   longname=xstrdup(fi.longname);
+   longname.set(fi.longname);
+}
+FileInfo::FileInfo(const char *n)
+{
+   Init();
+   SetName(n);
+}
+
+FileInfo::~FileInfo()
+{
 }
 
 #ifndef S_ISLNK
@@ -878,7 +823,9 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    t = NEXT_TOKEN;
    if(!t)
       ERR;
+   date.tm_isdst=-1;
    date.tm_hour=date.tm_min=0;
+   date.tm_sec=30;
    int prec=30;
    if(strlen(t)==5)
    {
@@ -894,12 +841,10 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
       /* We don't know the hour.  Set it to something other than 0, or
        * DST -1 will end up changing the date. */
       date.tm_hour = 12;
-      prec=12*60*60;
+      date.tm_sec=0;
+      prec=12*HOUR;
    }
 
-   date.tm_isdst=-1;
-   date.tm_sec=30;
-
    fi->SetDate(mktime_from_tz(&date,tz),prec);
 
    char *name=strtok_r(NULL,"",&next);
@@ -931,14 +876,6 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
 }
 
 
-FileInfo::~FileInfo()
-{
-   xfree(name);
-   xfree(symlink);
-   xfree(data);
-   xfree(longname);
-}
-
 int FileSet::Have() const
 {
    int bits=0;
@@ -951,11 +888,12 @@ int FileSet::Have() const
 
 static bool fnmatch_dir(const char *pattern,const FileInfo *file)
 {
-   char *name=file->name;
+   const char *name=file->name;
    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
    {
-      name=alloca_strdup2(name,1);
-      strcat(name,"/");
+      char *n=alloca_strdup2(name,1);
+      strcat(n,"/");
+      name=n;
    }
    return fnmatch(pattern,name,FNM_PATHNAME|FNM_CASEFOLD);
 }
@@ -976,8 +914,8 @@ void FileSet::SortByPatternList(const char *list_c)
 
 void FileInfo::MakeLongName()
 {
-   longname=(char*)xrealloc(longname,80+xstrlen(name)+xstrlen(symlink));
-   char filetype_c='-';
+   char filetype_s[2]="-";
+   char &filetype_c=filetype_s[0];
    switch(filetype)
    {
    case NORMAL:	   break;
@@ -987,31 +925,38 @@ void FileInfo::MakeLongName()
    }
    int mode1=(defined&MODE?mode:
       (filetype_c=='d'?0755:(filetype_c=='l'?0777:0644)));
-   sprintf(longname,"%c%s  ",filetype_c,format_perms(mode1));
+
    char usergroup[33];
    usergroup[0]=0;
    if(defined&(USER|GROUP))
       sprintf(usergroup,"%.16s%s%.16s",defined&USER?user:"?",
 		  defined&GROUP?"/":"",defined&GROUP?group:"");
-   char size_str[20];
-   strcpy(size_str,"-");
-   if(defined&SIZE)
-      sprintf(size_str,"%lld",(long long)size);
+
    int w=20-strlen(usergroup);
    if(w<1)
       w=1;
-   sprintf(longname+strlen(longname),"%s %*s ",usergroup,w,size_str);
+   char size_str[20];
+   if(defined&SIZE)
+      sprintf(size_str,"%*lld",w,(long long)size);
+   else
+      sprintf(size_str,"%*s",w,"-");
+
    const char *date_str="-";
    if(defined&DATE)
       date_str=TimeDate(date).IsoDateTime();
-   sprintf(longname+strlen(longname),"%s %s",date_str,name);
+
+   longname.vset(filetype_s,format_perms(mode1),"  ",usergroup," ",size_str,
+      " ",date_str," ",name.get(),NULL);
+
    if(defined&SYMLINK_DEF)
-      sprintf(longname+strlen(longname)," -> %s",symlink);
+      longname.vappend(" -> ",symlink.get(),NULL);
 }
 
-int FileSet::EstimateMemory() const
+size_t FileSet::EstimateMemory() const
 {
-   int size=sizeof(FileSet)+sizeof(FileInfo*)*allocated;
+   size_t size=sizeof(FileSet)
+      +files.count()*files.get_element_size()
+      +sorted.count()*sorted.get_element_size();
    for(int i=0; i<fnum; i++)
    {
       size+=sizeof(FileInfo);
