@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FindJob.cc,v 1.37 2006/03/15 11:16:08 lav Exp $ */
+/* $Id: FindJob.cc,v 1.44 2007/09/10 10:08:30 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -32,8 +32,10 @@
 #include "PatternSet.h"
 #include "buffer_std.h"
 
-#define top (*stack[stack_ptr])
+#define top (*stack.last())
+#define stack_ptr (stack.count()-1)
 #define super SessionJob
+#define orig_session super::session
 
 int FinderJob::Do()
 {
@@ -50,10 +52,10 @@ int FinderJob::Do()
 	 ParsedURL u(dir,true);
 	 if(u.proto)
 	 {
-	    session=FileAccess::New(&u);
+	    session=my_session=FileAccess::New(&u);
 	    session->SetPriority(fg?1:0);
 	    init_dir=session->GetCwd();
-	    Down(u.path?u.path:init_dir.path);
+	    Down(u.path?u.path.get():init_dir.path.get());
 	 }
       }
 
@@ -98,7 +100,6 @@ int FinderJob::Do()
       {
 	 if(!quiet)
 	    eprintf("%s: %s\n",op,li->ErrorText());
-	 Delete(li);
 	 li=0;
 	 errors++;
 	 depth_done=true;
@@ -112,7 +113,6 @@ int FinderJob::Do()
       Push(li->GetResult());
       top.fset->rewind();
 
-      Delete(li);
       li=0;
       state=LOOP;
       m=MOVED;
@@ -210,7 +210,7 @@ void FinderJob::Up()
     * stack[1] is the contents (ls dir); don't exit for the first. */
    if(stack_ptr)
       Exit();
-   delete stack[stack_ptr--];
+   stack.chop();
    if(stack_ptr==-1)
       goto done;
    depth_done=true;
@@ -221,17 +221,10 @@ void FinderJob::Push(FileSet *fset)
 {
    const char *old_path=0;
    if(stack_ptr>=0)
-      old_path=top.path;
-
-   stack_ptr++;
-   if(stack_allocated<=stack_ptr)
    {
-      stack_allocated=stack_ptr+8;
-      stack=(place**)xrealloc(stack,sizeof(*stack)*stack_allocated);
-   }
-
-   if(stack_ptr != 0)
+      old_path=top.path;
       fset->ExcludeDots(); /* don't need . and .. (except for stack[0]) */
+   }
 
    const char *new_path="";
    if(old_path) // the first path will be empty
@@ -241,31 +234,19 @@ void FinderJob::Push(FileSet *fset)
     * on the filename portion only */
    if(exclude)
       fset->Exclude(0, exclude);
-   stack[stack_ptr]=new place(new_path,fset);
+   stack.append(new place(new_path,fset));
 
    /* give a chance to operate on the list as a whole, and
     * possibly sort it */
    ProcessList(fset);
 }
 
-FinderJob::place::place(const char *p,FileSet *f)
-{
-   path=xstrdup(p);
-   fset=f;
-}
-FinderJob::place::~place()
-{
-   xfree(path);
-   if(fset) delete fset;
-}
-
 void FinderJob::Down(const char *p)
 {
 #ifdef FIND_DEBUG
-   printf("Down(%s)\n",p);
+   printf("Down(%s)\n",p.get());
 #endif
-   xfree(dir);
-   dir=xstrdup(p);
+   dir.set(p);
    state=START_INFO;
 }
 
@@ -276,17 +257,10 @@ FinderJob::prf_res FinderJob::ProcessFile(const char *d,const FileInfo *f)
 
 void FinderJob::Init()
 {
-   orig_session=0;
-
    op="find";
-   dir=0;
    errors=0;
    li=0;
 
-   stack_allocated=0;
-   stack_ptr=-1;
-   stack=0;
-
    show_sl=true;
 
    depth_first=false; // useful for rm -r
@@ -304,17 +278,16 @@ void FinderJob::Init()
 }
 
 FinderJob::FinderJob(FileAccess *s)
-   : SessionJob(s), orig_init_dir(s->GetCwd()), init_dir(s->GetCwd())
+   : SessionJob(s), orig_init_dir(orig_session->GetCwd()),
+     session(orig_session), init_dir(session->GetCwd())
 {
    Init();
-   orig_session=session;
 }
 
 void FinderJob::NextDir(const char *d)
 {
    if(session!=orig_session)
    {
-      Reuse(session);
       session=orig_session;
       init_dir=orig_init_dir;
    }
@@ -324,28 +297,17 @@ void FinderJob::NextDir(const char *d)
 
 FinderJob::~FinderJob()
 {
-   while(stack_ptr>=0)
-      Up();
-   if(orig_session!=session)
-      Reuse(orig_session);
-   xfree(stack);
-   delete exclude;
-   xfree(dir);
-   Delete(li);
 }
 
-void FinderJob::ShowRunStatus(StatusLine *sl)
+void FinderJob::ShowRunStatus(const SMTaskRef<StatusLine>& sl)
 {
    if(!show_sl)
       return;
 
-   char *path=0;
    switch(state)
    {
    case INFO:
-      if(stack_ptr>=0)
-	 path=top.path;
-      sl->Show("%s: %s",dir_file(path,dir),li->Status());
+      sl->Show("%s: %s",dir_file(stack_ptr>=0?top.path.get():0,dir),li->Status());
       break;
    case WAIT:
       Job::ShowRunStatus(sl);
@@ -360,13 +322,10 @@ void FinderJob::PrintStatus(int v,const char *prefix)
 {
    SessionJob::PrintStatus(v,prefix);
 
-   char *path=0;
    switch(state)
    {
    case INFO:
-      if(stack_ptr>=0)
-	 path=top.path;
-      printf("\t%s: %s\n",dir_file(path,dir),li->Status());
+      printf("\t%s: %s\n",dir_file(stack_ptr>=0?top.path.get():0,dir),li->Status());
       break;
    case WAIT:
       break;
@@ -379,12 +338,12 @@ void FinderJob::Fg()
 {
    super::Fg();
    if(orig_session!=session)
-      orig_session->SetPriority(1);
+      session->SetPriority(1);
 }
 void FinderJob::Bg()
 {
    if(orig_session!=session)
-      orig_session->SetPriority(0);
+      session->SetPriority(0);
    super::Bg();
 }
 
@@ -431,12 +390,6 @@ FinderJob_List::FinderJob_List(FileAccess *s,ArgV *a,FDStream *o)
    ValidateArgs();
 }
 
-FinderJob_List::~FinderJob_List()
-{
-   Delete(buf);
-   delete args;
-}
-
 void FinderJob_List::Finish()
 {
    const char *d=args->getnext();
@@ -446,107 +399,3 @@ void FinderJob_List::Finish()
    }
    NextDir(d);
 }
-
-#if 0 // unused
-// FinderJob_Cmd implementation
-// process directory tree
-#undef super
-#define super FinderJob
-
-FinderJob::prf_res FinderJob_Cmd::ProcessFile(const char *d,const FileInfo *f)
-{
-#define ISDIR  ((f->defined&f->TYPE) && f->filetype==f->DIRECTORY)
-#define ISLINK ((f->defined&f->TYPE) && f->filetype==f->SYMLINK)
-
-   FileAccess *s=session->Clone();
-
-   CmdExec *exec=new CmdExec(s);
-   exec->SetParentFg(this);
-   exec->SetCWD(saved_cwd);
-
-   char *file=f->name;
-
-   switch(cmd)
-   {
-   case RM:
-      if(ISDIR)
-      {
-	 exec->FeedCmd("rmdir ");
-	 if(quiet)
-	    exec->FeedCmd("-f ");
-	 exec->FeedCmd("-- ");
-	 exec->FeedQuoted(file);
-	 exec->FeedCmd("\n");
-      }
-      else
-      {
-	 exec->FeedCmd("rm ");
-	 if(quiet)
-	    exec->FeedCmd("-f ");
-	 exec->FeedCmd("-- ");
-	 exec->FeedQuoted(file);
-	 exec->FeedCmd("\n");
-      }
-      break;
-   case GET:
-#if 0
-      if(ISDIR)
-      {
-// 	 mkdir(dir_file(saved_cwd,
-      }
-      else if(ISLINK)
-      {
-      }
-      else
-      {
-      }
-#endif
-      break;
-   }
-   AddWaiting(exec);
-   return PRF_WAIT;
-
-#undef ISDIR
-#undef ISLINK
-}
-
-FinderJob_Cmd::FinderJob_Cmd(FileAccess *s,ArgV *a,cmd_t c)
-   : FinderJob(s)
-{
-   cmd=c;
-   args=a;
-   op=args->a0();
-   use_cache=false;
-   if(cmd==RM)
-      depth_first=true;
-   saved_cwd=xgetcwd();
-   removing_last=false;
-   NextDir(a->getcurr());
-}
-FinderJob_Cmd::~FinderJob_Cmd()
-{
-   xfree(saved_cwd);
-   delete args;
-   while(waiting_num>0)
-   {
-      Job *r=waiting[0];
-      for(int k=0; k<r->waiting_num; k++)
-	 r->waiting[k]->jobno=-1; // prevent reparenting
-      RemoveWaiting(r);
-      Delete(r);
-   }
-}
-
-void FinderJob_Cmd::Finish()
-{
-   char *d=args->getnext();
-   if(!d)
-      return;
-   FinderJob::NextDir(d);
-}
-
-int FinderJob_Cmd::Done()
-{
-   return FinderJob::Done() && args->getcurr()==0;
-}
-#endif
