@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
  */
 
 #include <config.h>
+#include <stddef.h>
 #include "Fish.h"
 #include "trio.h"
 #include <unistd.h>
@@ -32,7 +33,7 @@
 #include "log.h"
 #include "ArgV.h"
 
-#define super NetAccess
+#define super SSH_Access
 
 #define max_buf 0x10000
 
@@ -72,7 +73,7 @@ int Fish::Do()
    // check if idle time exceeded
    if(mode==CLOSED && send_buf && idle_timer.Stopped())
    {
-      DebugPrint("---- ",_("Closing idle connection"),1);
+      LogNote(1,_("Closing idle connection"));
       Disconnect();
       return m;
    }
@@ -88,7 +89,8 @@ int Fish::Do()
       Disconnect();
       return MOVED;
    }
-   m|=HandleReplies();
+   if(state!=CONNECTING_1)
+      m|=HandleReplies();
 
    if(Error())
       return m;
@@ -130,9 +132,7 @@ int Fish::Do()
 	 return MOVED;
 
       const char *shell=Query("shell",hostname);
-      char *init=alloca_strdup2("echo FISH:;",strlen(shell));
-      strcat(init,shell);
-
+      const char *init=xstring::cat("echo FISH:;",shell,NULL);
       const char *prog=Query("connect-program",hostname);
       if(!prog || !prog[0])
 	 prog="ssh -a -x";
@@ -152,12 +152,10 @@ int Fish::Do()
       }
       cmd->Add(hostname);
       cmd->Add(init);
-      {
-      char *cmd_str=cmd->Combine(0);
-      Log::global->Format(9,"---- %s (%s)\n",_("Running connect program"),cmd_str);
-      xfree(cmd_str);
-      }
+      xstring_ca cmd_str(cmd->Combine(0));
+      Log::global->Format(9,"---- %s (%s)\n",_("Running connect program"),cmd_str.get());
       ssh=new PtyShell(cmd);
+      ssh->UsePipes();
       state=CONNECTING;
       timeout_timer.Reset();
       m=MOVED;
@@ -174,15 +172,15 @@ int Fish::Do()
 	 TimeoutS(1);
 	 return m;
       }
-      ssh->Kill(SIGCONT);
-      send_buf=new IOBufferFDStream(ssh,IOBuffer::PUT);
-      ssh=0;
-      recv_buf=new IOBufferFDStream(new FDStream(fd,"pseudo-tty"),IOBuffer::GET);
+      MakePtyBuffers();
       set_real_cwd("~");
       state=CONNECTING_1;
       m=MOVED;
 
    case CONNECTING_1:
+      m|=HandleSSHMessage();
+      if(state!=CONNECTING_1)
+	 return MOVED;
       if(!received_greeting)
 	 return m;
 
@@ -194,8 +192,8 @@ int Fish::Do()
       }
 
       Send("#FISH\n"
-	   "exec 2>&1;echo;start_fish_server;"
 	   "TZ=GMT;export TZ;LC_ALL=C;export LC_ALL;"
+	   "exec 2>&1;echo;start_fish_server;"
 	   "echo '### 200'\n");
       PushExpect(EXPECT_FISH);
       Send("#VER 0.0.2\n"
@@ -214,26 +212,26 @@ int Fish::Do()
       if(mode==CLOSED)
 	 return m;
       if(home.path==0 && !RespQueueIsEmpty())
-	 goto usual_return;
+	 break;
       ExpandTildeInCWD();
       if(mode!=CHANGE_DIR && xstrcmp(cwd,real_cwd))
       {
-	 if(path_queue_len==0 || strcmp(path_queue[path_queue_len-1],cwd))
+	 if(xstrcmp(path_queue.LastString(),cwd))
 	 {
 	    Send("#CWD %s\n"
-		 "cd %s; echo '### 000'\n",cwd.path,shell_encode(cwd));
+		 "cd %s; echo '### 000'\n",cwd.path.get(),shell_encode(cwd));
 	    PushExpect(EXPECT_CWD);
 	    PushDirectory(cwd);
 	 }
 	 if(!RespQueueIsEmpty())
-	    goto usual_return;
+	    break;
       }
       SendMethod();
       if(mode==LONG_LIST || mode==LIST || mode==QUOTE_CMD)
       {
 	 state=FILE_RECV;
 	 m=MOVED;
-	 goto usual_return;
+	 break;
       }
       state=WAITING;
       m=MOVED;
@@ -258,7 +256,7 @@ int Fish::Do()
 	    state=DONE;
 	 m=MOVED;
       }
-      goto usual_return;
+      break;
    case FILE_RECV:
       if(recv_buf->Size()>=rate_limit->BytesAllowedToGet())
       {
@@ -279,9 +277,8 @@ int Fish::Do()
       break;
    case FILE_SEND:
    case DONE:
-      goto usual_return;
+      break;
    }
-usual_return:
    if(m==MOVED)
       return MOVED;
    if(send_buf)
@@ -290,26 +287,17 @@ usual_return:
       timeout_timer.Reset(recv_buf->EventTime());
    if(CheckTimeout())
       return MOVED;
-// notimeout_return:
-   if(m==MOVED)
-      return MOVED;
    return m;
 }
 
 void Fish::MoveConnectionHere(Fish *o)
 {
-   send_buf=o->send_buf; o->send_buf=0;
-   recv_buf=o->recv_buf; o->recv_buf=0;
-   rate_limit=o->rate_limit; o->rate_limit=0;
-   path_queue=o->path_queue; o->path_queue=0;
-   path_queue_len=o->path_queue_len; o->path_queue_len=0;
-   RespQueue=o->RespQueue; o->RespQueue=0;
-   RQ_alloc=o->RQ_alloc; o->RQ_alloc=0;
-   RQ_head=o->RQ_head; o->RQ_head=0;
-   RQ_tail=o->RQ_tail; o->RQ_tail=0;
+   super::MoveConnectionHere(o);
+   rate_limit=o->rate_limit.borrow();
+   path_queue.MoveHere(o->path_queue);
+   RespQueue.move_here(o->RespQueue);
    timeout_timer.Reset(o->timeout_timer);
    set_real_cwd(o->real_cwd);
-   o->set_real_cwd(0);
    state=CONNECTED;
    o->Disconnect();
    if(!home)
@@ -319,49 +307,23 @@ void Fish::MoveConnectionHere(Fish *o)
 
 void Fish::Disconnect()
 {
-   if(send_buf)
-      DebugPrint("---- ",_("Disconnecting"),9);
-   Delete(send_buf); send_buf=0;
-   Delete(recv_buf); recv_buf=0;
-   delete ssh; ssh=0;
+   super::Disconnect();
    EmptyRespQueue();
    EmptyPathQueue();
    state=DISCONNECTED;
    if(mode==STORE)
       SetError(STORE_FAILED,0);
-   received_greeting=false;
-   password_sent=0;
-   xfree(home_auto); home_auto=0;
-   home_auto=xstrdup(FindHomeAuto());
-}
-
-void Fish::EmptyPathQueue()
-{
-   for(int i=0; i<path_queue_len; i++)
-      xfree(path_queue[i]);
-   path_queue_len=0;
+   home_auto.set(FindHomeAuto());
 }
 
 void Fish::Init()
 {
    state=DISCONNECTED;
-   send_buf=0;
-   recv_buf=0;
-   ssh=0;
    max_send=0;
-   line=0;
-   message=0;
-   RespQueue=0;
-   RQ_alloc=0;
-   RQ_head=RQ_tail=0;
    eof=false;
-   path_queue=0;
-   path_queue_len=0;
-   received_greeting=false;
-   password_sent=0;
 }
 
-Fish::Fish()
+Fish::Fish() : SSH_Access("FISH:")
 {
    Init();
    Reconfig(0);
@@ -370,12 +332,6 @@ Fish::Fish()
 Fish::~Fish()
 {
    Disconnect();
-   xfree(line);
-   xfree(message);
-   EmptyRespQueue();
-   xfree(RespQueue);
-   EmptyPathQueue();
-   xfree(path_queue);
 }
 
 Fish::Fish(const Fish *o) : super(o)
@@ -414,25 +370,10 @@ void Fish::Close()
 void Fish::Send(const char *format,...)
 {
    va_list va;
-   char *str;
-
-   static int max_send=256;
-   for(;;)
-   {
-      va_start(va,format);
-      str=string_alloca(max_send);
-      int res=vsnprintf(str,max_send,format,va);
-      va_end(va);
-      if(res>=0 && res<max_send)
-      {
-	 if(res<max_send/16)
-	    max_send/=2;
-	 break;
-      }
-      max_send*=2;
-   }
-
-   DebugPrint("---> ",str,5);
+   va_start(va,format);
+   xstring& str=xstring::vformat(format,va);
+   va_end(va);
+   LogSend(5,str);
    send_buf->Put(str);
 }
 
@@ -581,7 +522,7 @@ void Fish::SendMethod()
       break;
    case QUOTE_CMD:
       Send("#EXEC %s\n"
-	   "%s; echo '### 200'\n",file,file);
+	   "%s; echo '### 200'\n",e,file.get());
       PushExpect(EXPECT_QUOTE);
       real_pos=0;
       break;
@@ -604,8 +545,28 @@ int Fish::ReplyLogPriority(int code)
 int Fish::HandleReplies()
 {
    int m=STALL;
-   if(recv_buf==0 || state==FILE_RECV)
+   if(recv_buf==0)
       return m;
+   if(state==FILE_RECV) {
+      const char *err=pty_recv_buf->Get();
+      if(err && err[0]) {
+	 const char *eol=strchr(err,'\n');
+	 if(eol) {
+	    xstring &e=xstring::get_tmp(err,eol-err);
+	    LogError(0,e);
+	    SetError(NO_FILE,e);
+	    if(pty_recv_buf)
+	       pty_recv_buf->Skip(eol-err+1);
+	    return MOVED;
+	 }
+      }
+      if(pty_recv_buf->Eof()) {
+	 Disconnect();
+	 return MOVED;
+      }
+   }
+   recv_buf->Put(pty_recv_buf->Get(),pty_recv_buf->Size()); // join the messages.
+   pty_recv_buf->Skip(pty_recv_buf->Size());
    if(recv_buf->Size()<5)
    {
    hup:
@@ -616,63 +577,30 @@ int Fish::HandleReplies()
       }
       if(recv_buf->Eof())
       {
-	 DebugPrint("**** ",_("Peer closed connection"),0);
-	 if(!RespQueueIsEmpty() && RespQueue[RQ_head]==EXPECT_CWD && message)
+	 LogError(0,_("Peer closed connection"));
+	 // Solaris' shell exists when is given with wrong directory
+	 if(!RespQueueIsEmpty() && RespQueue[0]==EXPECT_CWD && message)
 	    SetError(NO_FILE,message);
 	 Disconnect();
 	 m=MOVED;
       }
       return m;
    }
+
    const char *b;
    int s;
    recv_buf->Get(&b,&s);
    const char *eol=(const char*)memchr(b,'\n',s);
    if(!eol)
    {
-      if(state==CONNECTING_1)
-      {
-	 const char *p="password:";
-	 const char *y="(yes/no)?";
-	 int p_len=strlen(p);
-	 int y_len=strlen(y);
-	 if(s>0 && b[s-1]==' ')
-	    s--;
-	 if(s>=p_len && !strncasecmp(b+s-p_len,p,p_len)
-	 || s>10 && !strncmp(b+s-2,"':",2))
-	 {
-	    if(!pass)
-	    {
-	       SetError(LOGIN_FAILED,_("Password required"));
-	       return MOVED;
-	    }
-	    if(password_sent>1)
-	    {
-	       SetError(LOGIN_FAILED,_("Login incorrect"));
-	       return MOVED;
-	    }
-	    recv_buf->Put("XXXX");
-	    send_buf->Put(pass);
-	    send_buf->Put("\n");
-	    password_sent++;
-	    return m;
-	 }
-	 if(s>=y_len && !strncasecmp(b+s-y_len,y,y_len))
-	 {
-	    recv_buf->Put("yes\n");
-	    send_buf->Put("yes\n");
-	    return m;
-	 }
-      }
       if(recv_buf->Eof() || recv_buf->Error())
 	 goto hup;
       return m;
    }
+
    m=MOVED;
    s=eol-b+1;
-   xfree(line);
-   line=(char*)xmemdup(b,s);
-   line[s-1]=0;
+   line.nset(b,s-1);
    recv_buf->Skip(s);
 
    int code=-1;
@@ -681,37 +609,26 @@ int Fish::HandleReplies()
 	 code=-1;
    }
 
-   DebugPrint("<--- ",line,ReplyLogPriority(code));
+   LogRecv(ReplyLogPriority(code),line);
    if(code==-1)
    {
-      if(!received_greeting && !strcmp(line,"FISH:"))
-      {
-	 received_greeting=true;
-	 return m;
-      }
       if(message==0)
-	 message=xstrdup(line);
+	 message.set(line);
       else
-      {
-	 message=(char*)xrealloc(message,xstrlen(message)+s+1);
-	 strcat(message,"\n");
-	 strcat(message,line);
-      }
+	 message.vappend("\n",line.get(),NULL);
       return m;
    }
 
    if(RespQueueIsEmpty())
    {
-      DebugPrint("**** ",_("extra server response"),3);
-      xfree(message);
-      message=0;
+      LogError(3,_("extra server response"));
+      message.set(0);
       return m;
    }
-   expect_t &e=RespQueue[RQ_head];
-   RQ_head++;
+   expect_t e=RespQueue.next();
 
    bool keep_message=false;
-   char *p;
+   xstring p;
 
    switch(e)
    {
@@ -722,15 +639,15 @@ int Fish::HandleReplies()
    case EXPECT_PWD:
       if(!message)
 	 break;
-      home_auto=xstrdup(message);
-      Log::global->Format(9,"---- home set to %s\n",home_auto);
+      home_auto.set(message);
+      Log::global->Format(9,"---- home set to %s\n",home_auto.get());
       PropagateHomeAuto();
       if(!home)
 	 set_home(home_auto);
       cache->SetDirectory(this, home, true);
       break;
    case EXPECT_CWD:
-      p=PopDirectory();
+      PopDirectory(&p);
       if(message==0)
       {
 	 set_real_cwd(p);
@@ -743,7 +660,6 @@ int Fish::HandleReplies()
       }
       else
 	 SetError(NO_FILE,message);
-      xfree(p);
       break;
    case EXPECT_RETR_INFO:
       if(message && is_ascii_digit(message[0]) && !strchr(message,':'))
@@ -824,32 +740,18 @@ int Fish::HandleReplies()
    }
 
    if(!keep_message)
-   {
-      xfree(message);
-      message=0;
-   }
+      message.set(0);
 
    return m;
 }
 void Fish::PushExpect(expect_t e)
 {
-   int newtail=RQ_tail+1;
-   if(newtail>RQ_alloc)
-   {
-      if(RQ_head-0<newtail-RQ_alloc)
-	 RespQueue=(expect_t*)
-	    xrealloc(RespQueue,(RQ_alloc=newtail+16)*sizeof(*RespQueue));
-      memmove(RespQueue,RespQueue+RQ_head,(RQ_tail-RQ_head)*sizeof(*RespQueue));
-      RQ_tail=0+(RQ_tail-RQ_head);
-      RQ_head=0;
-      newtail=RQ_tail+1;
-   }
-   RespQueue[RQ_tail]=e;
-   RQ_tail=newtail;
+   RespQueue.push(e);
 }
 void Fish::CloseExpectQueue()
 {
-   for(int i=RQ_head; i<RQ_tail; i++)
+   int count=RespQueue.count();
+   for(int i=0; i<count; i++)
    {
       switch(RespQueue[i])
       {
@@ -875,19 +777,6 @@ void Fish::CloseExpectQueue()
    }
 }
 
-void Fish::PushDirectory(const char *p)
-{
-   path_queue=(char**)xrealloc(path_queue,++path_queue_len*sizeof(*path_queue));
-   path_queue[path_queue_len-1]=xstrdup(p);
-}
-char *Fish::PopDirectory()
-{
-   assert(path_queue_len>0);
-   char *p=path_queue[0];
-   memmove(path_queue,path_queue+1,--path_queue_len*sizeof(*path_queue));
-   return p; // caller should free it.
-}
-
 const char *memstr(const char *mem,size_t len,const char *str)
 {
    size_t str_len=strlen(str);
@@ -1152,6 +1041,11 @@ void Fish::Reconfig(const char *name)
 	 send_buf->SetTranslation(charset,false);
 	 recv_buf->SetTranslation(charset,true);
       }
+      else
+      {
+	 send_buf->SetTranslator(0);
+	 recv_buf->SetTranslator(0);
+      }
    }
 }
 
@@ -1164,7 +1058,7 @@ FileAccess *Fish::New() { return new Fish(); }
 
 DirList *Fish::MakeDirList(ArgV *args)
 {
-   return new FishDirList(args,this);
+   return new FishDirList(this,args);
 }
 #include "FileGlob.h"
 Glob *Fish::MakeGlob(const char *pattern)
@@ -1211,7 +1105,7 @@ int FishDirList::Do()
       else
       {
 	 session->Open(pattern,FA::LONG_LIST);
-	 ((Fish*)session)->DontEncodeFile();
+	 session.Cast<Fish>()->DontEncodeFile();
 	 ubuf=new IOBufferFileAccess(session);
 	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
 	    ubuf->Save(FileAccess::cache->SizeLimit());
@@ -1245,20 +1139,6 @@ int FishDirList::Do()
    return m;
 }
 
-FishDirList::FishDirList(ArgV *a,FileAccess *fa)
-   : DirList(a)
-{
-   session=fa;
-   ubuf=0;
-   pattern=args->Combine(1);
-}
-
-FishDirList::~FishDirList()
-{
-   Delete(ubuf);
-   xfree(pattern);
-}
-
 const char *FishDirList::Status()
 {
    static char s[256];
