@@ -18,16 +18,15 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpDirList.h,v 1.9 2005/06/22 08:04:19 lav Exp $ */
+/* $Id: FtpDirList.h,v 1.11 2007/04/23 07:11:45 lav Exp $ */
 
 #ifndef FTPDIRLIST_H
 #define FTPDIRLIST_H
 
 class FtpDirList : public DirList
 {
-   FileAccess *session;
-   IOBuffer *ubuf;
-   char *pattern;
+   SMTaskRef<IOBuffer> ubuf;
+   xstring_ca pattern;
 
    bool TryEPLF(const char *line,int len);
    bool TryMLSD(const char *line,int len);
@@ -36,8 +35,8 @@ class FtpDirList : public DirList
    void FormatGeneric(class FileInfo *);
 
 public:
-   FtpDirList(ArgV *a,FileAccess *fa);
-   ~FtpDirList();
+   FtpDirList(FileAccess *s,ArgV *a)
+      : DirList(s,a), pattern(args->Combine(1)) {}
    const char *Status();
    int Do();
 
