@@ -18,12 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: GetJob.cc,v 1.25 2006/06/30 14:10:27 lav Exp $ */
+/* $Id: GetJob.cc,v 1.31 2008/05/22 13:20:36 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <stddef.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -56,12 +57,12 @@ int   GetJob::Do()
    return m;
 }
 
-FileCopyPeer *GetJob::NoProtoSrcLocal(const char *src)
+FileCopyPeer *GetJob::SrcLocal(const char *src)
 {
    const char *f=(cwd && src[0]!='/') ? dir_file(cwd,src) : src;
    return FileCopyPeerFDStream::NewGet(f);
 }
-FileCopyPeer *GetJob::NoProtoDstLocal(const char *dst)
+FileCopyPeer *GetJob::DstLocal(const char *dst)
 {
    int flags=O_WRONLY|O_CREAT|(truncate_target_first?O_TRUNC:0);
    dst=expand_home_relative(dst);
@@ -81,75 +82,69 @@ FileCopyPeer *GetJob::NoProtoDstLocal(const char *dst)
 	       count++;
 	       return 0;
 	    }
-	    backup_file=xstrdup(f,+1);
-	    strcat(backup_file,"~");
+	    backup_file.vset(f,"~",NULL);
 	    if(rename(f,backup_file)!=0)
-	    {
-	       xfree(backup_file);
-	       backup_file=0;
-	    }
+	       backup_file.set(0);
 	    else
-	    {
 	       file_mode=st.st_mode;
-	    }
 	 }
       }
    }
    local=new FileStream(f,flags); // local is for pget.
    FileCopyPeerFDStream *p=new FileCopyPeerFDStream(local,FileCopyPeer::PUT);
-   p->DontDeleteStream();
    return p;
 }
-FileCopyPeer *GetJob::NoProtoPeer(FileAccess *session,const char *src,FA::open_mode mode)
-{
-   FileCopyPeerFA *peer=new FileCopyPeerFA(session,src,mode);
-   if(this->session==session)
-      peer->DontReuseSession();
-   return peer;
-}
 
+bool GetJob::IsRemoteNonURL(const ParsedURL &url,FA::open_mode mode)
+{
+   // store & put || !store & get
+   return (!url.proto && ((mode==FA::STORE)^!reverse));
+}
+bool GetJob::IsLocalNonURL(const ParsedURL &url,FA::open_mode mode)
+{
+   // store & get || !store & put
+   return (!url.proto && ((mode==FA::STORE)^reverse));
+}
+bool GetJob::IsLocal(const ParsedURL &url)
+{
+   return !url.proto || !strcasecmp(url.proto,"file");
+}
+// create copy peer from a cloned session
 FileCopyPeer *GetJob::CreateCopyPeer(FileAccess *session,const char *path,FA::open_mode mode)
 {
-   ParsedURL url(path);
-   bool is_local;
-   if(!url.proto)
-   {
-      // store & put || !store & get
-      if(mode==FA::STORE ^ !reverse)
-	 return NoProtoPeer(session,path,mode);
-      is_local=true;
-   }
-   else
-   {
-      path=url.path;
-      is_local=!strcasecmp(url.proto,"file");
-   }
-
-   if(session!=this->session)
-      Delete(session);	// delete cloned session.
-
-   if(is_local)
-      return (mode==FA::STORE)
-	 ? NoProtoDstLocal(path)
-	 : NoProtoSrcLocal(path);
-
+   ParsedURL url(path,true);
+   if(IsRemoteNonURL(url,mode))
+      return new FileCopyPeerFA(session,path,mode);
+   Delete(session);	// delete cloned session.
+   return CreateCopyPeer(url,path,mode);
+}
+// create copy peer using a session reference
+FileCopyPeer *GetJob::CreateCopyPeer(const FileAccessRef& session,const char *path,FA::open_mode mode)
+{
+   ParsedURL url(path,true);
+   if(IsRemoteNonURL(url,mode))
+      return new FileCopyPeerFA(session,path,mode);
+   return CreateCopyPeer(url,path,mode);
+}
+FileCopyPeer *GetJob::CreateCopyPeer(const ParsedURL &url,const char *path,FA::open_mode mode)
+{
+   if(IsLocalNonURL(url,mode))
+      return CreateCopyPeer(path,mode);
+   if(IsLocal(url))
+      return CreateCopyPeer(url.path,mode);
    return new FileCopyPeerFA(&url,mode);
 }
+FileCopyPeer *GetJob::CreateCopyPeer(const char *path,FA::open_mode mode)
+{
+   return mode==FA::STORE ? DstLocal(path) : SrcLocal(path);
+}
 
 void GetJob::NextFile()
 {
 try_next:
    file_mode=NO_MODE;
-   if(backup_file)
-   {
-      xfree(backup_file);
-      backup_file=0;
-   }
-   if(local)
-   {
-      delete local;
-      local=0;
-   }
+   backup_file.set(0);
+   local=0;
 
    if(!args)
       return;
@@ -182,8 +177,7 @@ void GetJob::RemoveBackupFile()
    if(backup_file)
    {
       remove(backup_file);
-      xfree(backup_file);
-      backup_file=0;
+      backup_file.set(0);
    }
 }
 
@@ -193,14 +187,6 @@ GetJob::GetJob(FileAccess *s,ArgV *a,bool c)
    delete_files=false;
    remove_target_first=false;
    truncate_target_first=!cont;
-   backup_file=0;
    file_mode=NO_MODE;
-   local=0;
    reverse=false;
 }
-GetJob::~GetJob()
-{
-   xfree(backup_file);
-   if(local)
-      delete local;
-}
