@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: GetJob.h,v 1.10 2004/02/17 10:45:57 lav Exp $ */
+/* $Id: GetJob.h,v 1.16 2007/12/28 14:55:13 lav Exp $ */
 
 #ifndef GETJOB_H
 #define GETJOB_H
@@ -27,25 +27,31 @@
 
 class GetJob : public CopyJobEnv
 {
+   FileCopyPeer *SrcLocal(const char *src);
+   FileCopyPeer *DstLocal(const char *dst);
+   FileCopyPeer *CreateCopyPeer(const ParsedURL &url,const char *path,FA::open_mode mode);
+   FileCopyPeer *CreateCopyPeer(const char *path,FA::open_mode mode);
+   bool IsRemoteNonURL(const ParsedURL &url,FA::open_mode mode);
+   bool IsLocalNonURL(const ParsedURL &url,FA::open_mode mode);
+   static bool IsLocal(const ParsedURL &url);
+
 protected:
+   FileCopyPeer *CreateCopyPeer(FileAccess *session,const char *path,FA::open_mode mode);
+   FileCopyPeer *CreateCopyPeer(const FileAccessRef& session,const char *path,FA::open_mode mode);
    void	 NextFile();
 
    bool delete_files;
    bool remove_target_first;
-   char *backup_file;
+   bool truncate_target_first;
+   xstring backup_file;
    mode_t file_mode;
-   FileStream *local;
+   Ref<FDStream> local;
    bool reverse;
 
    void RemoveBackupFile();
 
-   FileCopyPeer *NoProtoSrc(const char *src,bool from_local);
-   FileCopyPeer *NoProtoDst(const char *dst,bool to_local);
-   FileCopyPeer *CreateCopyPeer(const char *path,FA::open_mode mode);
-
 public:
    GetJob(FileAccess *s,ArgV *a,bool c=false);
-   ~GetJob();
 
    int	 Do();
 
