@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.cc,v 1.45 2007/10/13 09:04:02 lav Exp $ */
+/* $Id: Job.cc,v 1.46 2008/08/05 12:10:10 lav Exp $ */
 
 #include <config.h>
 #include <stdlib.h>
@@ -156,7 +156,7 @@ void Job::AddWaiting(Job *j)
    assert(FindWhoWaitsFor(j)==0);
    waiting.append(j);
 }
-void Job::RemoveWaiting(Job *j)
+void Job::RemoveWaiting(const Job *j)
 {
    for(int i=0; i<waiting_num; i++)
    {
