@@ -18,13 +18,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LsCache.cc,v 1.38 2006/06/13 14:35:38 lav Exp $ */
+/* $Id: LsCache.cc,v 1.44 2007/08/29 11:20:14 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
 #include "FileAccess.h"
 #include "LsCache.h"
-#include "xmalloc.h"
 #include "plural.h"
 #include "misc.h"
 
@@ -36,11 +35,10 @@ int LsCacheEntry::EstimateSize() const
    return size;
 }
 LsCacheEntryLoc::LsCacheEntryLoc(const FileAccess *p_loc,const char *a,int m)
+   : arg(a), mode(m)
 {
    loc=p_loc->Clone();
    loc->Suspend();
-   arg=xstrdup(a);
-   mode=m;
 }
 const char *LsCacheEntryLoc::GetClosure() const
 {
@@ -48,32 +46,27 @@ const char *LsCacheEntryLoc::GetClosure() const
 }
 LsCacheEntryData::LsCacheEntryData(int e,const char *d,int l,const FileSet *fs)
 {
-   data=0;
-   afset=0;
    SetData(e,d,l,fs);
 }
 
-LsCacheEntry::LsCacheEntry(CacheEntry *n,const FileAccess *p_loc,const char *a,int m,int e,const char *d,int l,const FileSet *fs)
-   : CacheEntry(n), LsCacheEntryLoc(p_loc,a,m), LsCacheEntryData(e,d,l,fs)
+LsCacheEntry::LsCacheEntry(const FileAccess *p_loc,const char *a,int m,int e,const char *d,int l,const FileSet *fs)
+   : LsCacheEntryLoc(p_loc,a,m), LsCacheEntryData(e,d,l,fs)
 {
    SetResource(e==FA::OK?"cache:expire":"cache:expire-negative",GetClosure());
 }
 
 void LsCacheEntryData::SetData(int e,const char *d,int l,const FileSet *fs)
 {
-   xfree(data);
-   delete afset;
-   err_code=e;
-   data=(char*)xmemdup(d,l);
-   data_len=l;
    afset=fs?new FileSet(fs):0;
+   data.nset(d,l);
+   err_code=e;
 }
 void LsCacheEntryData::GetData(int *e,const char **d,int *l,const FileSet **fs)
 {
    if(d && l)
    {
       *d=data;
-      *l=data_len;
+      *l=data.length();
    }
    if(fs)
       *fs=afset;
@@ -81,14 +74,14 @@ void LsCacheEntryData::GetData(int *e,const char **d,int *l,const FileSet **fs)
 }
 bool LsCacheEntryLoc::Matches(const FileAccess *p_loc,const char *a,int m)
 {
-   return (m==-1 || mode==m) && !xstrcmp(arg,a) && p_loc->SameLocationAs(loc);
+   return (m==-1 || mode==m) && arg.eq(a) && p_loc->SameLocationAs(loc);
 }
 
 ResDecl res_cache_empty_listings("cache:cache-empty-listings","no",ResMgr::BoolValidate,0);
 ResDecl res_cache_enable("cache:enable","yes",ResMgr::BoolValidate,0);
 ResDecl res_cache_expire("cache:expire","60m",ResMgr::TimeIntervalValidate,0);
 ResDecl res_cache_expire_neg("cache:expire-negative","1m",ResMgr::TimeIntervalValidate,0);
-ResDecl res_cache_size  ("cache:size","1048576",ResMgr::UNumberValidate,ResMgr::NoClosure);
+ResDecl res_cache_size  ("cache:size","16777216",ResMgr::UNumberValidate,ResMgr::NoClosure);
 
 LsCache::LsCache() : Cache(&res_cache_size,&res_cache_enable) {}
 
@@ -109,7 +102,7 @@ void LsCache::Add(const FileAccess *p_loc,const char *a,int m,int e,const char *
    {
       if(!IsEnabled(p_loc->GetHostName()))
 	 return;
-      chain=new LsCacheEntry(chain,p_loc,a,m,e,d,l,fs);
+      AddCacheEntry(new LsCacheEntry(p_loc,a,m,e,d,l,fs));
    }
    else
    {
@@ -175,21 +168,8 @@ const FileSet *LsCacheEntryData::GetFileSet(const FileAccess *parser)
       return afset;
    if(err_code!=FA::OK)
       return 0;
-   return afset=parser->ParseLongList(data, data_len);
-}
-
-LsCacheEntryLoc::~LsCacheEntryLoc()
-{
-   loc->DeleteLater();
-   xfree(arg);
-}
-LsCacheEntryData::~LsCacheEntryData()
-{
-   xfree(data);
-   delete afset;
-}
-LsCacheEntry::~LsCacheEntry()
-{
+   afset=parser->ParseLongList(data, data.length());
+   return afset;
 }
 
 void LsCache::List()
@@ -211,19 +191,17 @@ void LsCache::List()
 
 void LsCache::Changed(change_mode m,const FileAccess *f,const char *dir)
 {
-   const char *fdir_c=dir_file(f->GetCwd(),dir);
-   char *fdir=alloca_strdup(fdir_c);
+   xstring fdir(dir_file(f->GetCwd(),dir));
    if(m==FILE_CHANGED)
       dirname_modify(fdir);
-   int fdir_len=strlen(fdir);
 
    LsCacheEntry *c=IterateFirst();
    while(c)
    {
-      FileAccess *sloc=c->loc;
+      const FileAccess *sloc=c->loc;
       if(f->SameLocationAs(sloc) || (f->SameSiteAs(sloc)
 	       && (m==TREE_CHANGED?
-		     !strncmp(fdir,dir_file(sloc->GetCwd(),c->arg),fdir_len)
+		     !strncmp(fdir,dir_file(sloc->GetCwd(),c->arg),fdir.length())
 		   : !strcmp (fdir,dir_file(sloc->GetCwd(),c->arg)))))
 	 c=IterateDelete();
       else
@@ -240,13 +218,11 @@ void LsCache::SetDirectory(const FileAccess *p_loc, const char *path, bool dir)
 
    FileAccess::Path new_cwd = p_loc->GetCwd();
    new_cwd.Change(path,!dir);
-   FileAccess *new_p_loc=p_loc->Clone();
+   SMTaskRef<FileAccess> new_p_loc(p_loc->Clone());
    new_p_loc->SetCwd(new_cwd);
 
    const char *entry = dir? "1":"0";
    LsCache::Add(new_p_loc,"",FileAccess::CHANGE_DIR, dir?FA::OK:FA::NO_FILE, entry, strlen(entry));
-
-   SMTask::Delete(new_p_loc);
 }
 
 /* This is a hint function. If file type is really needed, use GetFileInfo
@@ -257,7 +233,7 @@ int LsCache::IsDirectory(const FileAccess *p_loc,const char *dir_c)
 {
    FileAccess::Path new_cwd = p_loc->GetCwd();
    new_cwd.Change(dir_c);
-   FileAccess *new_p_loc=p_loc->Clone();
+   SMTaskRef<FileAccess> new_p_loc(p_loc->Clone());
    new_p_loc->SetCwd(new_cwd);
 
    int ret = -1;
@@ -272,32 +248,25 @@ int LsCache::IsDirectory(const FileAccess *p_loc,const char *dir_c)
    if(Find(new_p_loc, "", FileAccess::CHANGE_DIR, &e, &buf_c,&bufsiz))
    {
       assert(bufsiz==1);
-      ret = (e==FA::OK);
-      goto leave;
+      return (e==FA::OK);
    }
 
    /* We know the path is a directory if we have a cache entry for it.  This is
     * true regardless of the list type.  (Unless it's a CHANGE_DIR entry; do this
     * test after the CHANGE_DIR check.) */
    if(Find(new_p_loc, "", FA::LONG_LIST, &e, 0,0))
-   {
-      ret = (e==FA::OK);
-      goto leave;
-   }
+      return(e==FA::OK);
    if(Find(new_p_loc, "", FA::MP_LIST, &e, 0,0))
-   {
-      ret = (e==FA::OK);
-      goto leave;
-   }
+      return(e==FA::OK);
    if(Find(new_p_loc, "", FA::LIST, &e, 0,0))
-   {
-      ret = (e==FA::OK);
-      goto leave;
-   }
+      return(e==FA::OK);
 
    /* We know this is a file or a directory if the dirname is cached and
     * contains the basename. */
    {
+      const char *bn=basename_ptr(new_cwd.path);
+      bn=alloca_strdup(bn); // save basename
+
       new_cwd.Change("..");
       new_p_loc->SetCwd(new_cwd);
 
@@ -306,13 +275,11 @@ int LsCache::IsDirectory(const FileAccess *p_loc,const char *dir_c)
 	 fs=FindFileSet(new_p_loc, "", FA::LONG_LIST);
       if(fs)
       {
-	 FileInfo *fi=fs->FindByName(basename_ptr(dir_c));
+	 FileInfo *fi=fs->FindByName(bn);
 	 if(fi && (fi->defined&fi->TYPE))
-	    ret = (fi->filetype == fi->DIRECTORY);
+	    return(fi->filetype == fi->DIRECTORY);
       }
    }
 
-leave:
-   SMTask::Delete(new_p_loc);
    return ret;
 }
