@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.161 2008/02/26 15:46:57 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.164 2008/07/23 08:46:10 lav Exp $ */
 
 #include <config.h>
 
@@ -41,6 +41,10 @@
 #include "pgetJob.h"
 #include "log.h"
 
+CDECL_BEGIN
+#include "lstat.h"
+CDECL_END
+
 #define set_state(s) do { state=(s); \
    Log::global->Format(11,"mirror(%p) enters state %s\n", this, #s); } while(0)
 #define waiting_num waiting.count()
@@ -1178,11 +1182,10 @@ extern "C" {
 }
 void MirrorJob::SetNewerThan(const char *f)
 {
-   time_t t=now;
-   t=get_date(f,&t);
-   if(t>0)
+   struct timespec ts;
+   if(get_date(&ts,f,0))
    {
-      newer_than=t;
+      newer_than=ts.tv_sec;
       return;
    }
    struct stat st;
@@ -1195,11 +1198,10 @@ void MirrorJob::SetNewerThan(const char *f)
 }
 void MirrorJob::SetOlderThan(const char *f)
 {
-   time_t t=now;
-   t=get_date(f,&t);
-   if(t>0)
+   struct timespec ts;
+   if(get_date(&ts,f,0))
    {
-      older_than=t;
+      older_than=ts.tv_sec;
       return;
    }
    struct stat st;
