@@ -22,23 +22,15 @@
 #define POLLVEC_H
 
 #include <sys/types.h>
+CDECL_BEGIN
+#include <poll.h>
+CDECL_END
 
-#ifdef HAVE_SYS_STROPTS_H
-# include <sys/stropts.h>
-#endif
-
-#ifdef HAVE_SYS_POLL_H
-# include <sys/poll.h>
-#else
-# include <poll.h>
-#endif
+#include "xarray.h"
 
 class PollVec
 {
-   struct pollfd *fds;
-   int fds_num;
-   int fds_allocated;
-
+   xarray<pollfd> fds;
    int timeout;
 
 public:
@@ -47,15 +39,15 @@ public:
 
    void	 Empty()
       {
-	 fds_num=0;
+	 fds.set_length(0);
 	 timeout=-1;
       }
    bool IsEmpty()
       {
-	 return fds_num==0 && timeout==-1;
+	 return fds.length()==0 && timeout==-1;
       }
    void	 Merge(const PollVec&);
-   void	 Block() const;
+   void	 Block();
 
    void SetTimeout(int t);
    void AddTimeout(int t);
@@ -63,8 +55,6 @@ public:
    void NoWait() { SetTimeout(0); }
 
    int GetTimeout() { return timeout; }
-
-   ~PollVec();
 };
 
 #endif /* POLLVEC_H */
