@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.cc,v 1.77 2008/02/26 15:46:57 lav Exp $ */
+/* $Id: SFtp.cc,v 1.80 2008/05/16 08:45:01 lav Exp $ */
 
 #include <config.h>
 #include "SFtp.h"
@@ -388,7 +388,12 @@ void SFtp::Packet::PackString(Buffer *b,const char *str,int len)
 SFtp::unpack_status_t SFtp::Packet::UnpackString(const Buffer *b,int *offset,int limit,xstring *str_out)
 {
    if(limit-*offset<4)
-      return b->Eof()?UNPACK_PREMATURE_EOF:UNPACK_NO_DATA_YET;
+   {
+      // We unpack strings when we have already received complete packet,
+      // so it is not possible to receive any more data.
+      LogError(2,"bad string in reply (truncated length field)");
+      return UNPACK_WRONG_FORMAT;
+   }
 
    int len=b->UnpackUINT32BE(*offset);
    if(len>limit-*offset-4)
@@ -441,8 +446,8 @@ SFtp::unpack_status_t SFtp::Packet::Unpack(const Buffer *b)
 
 SFtp::unpack_status_t SFtp::UnpackPacket(Buffer *b,SFtp::Packet **p)
 {
-   *p=0;
    Packet *&pp=*p;
+   pp=0;
 
    Packet probe;
    unpack_status_t res=probe.Unpack(b);
@@ -517,8 +522,8 @@ SFtp::unpack_status_t SFtp::UnpackPacket(Buffer *b,SFtp::Packet **p)
 	 ;
       }
       probe.DropData(b);
-      delete *p;
-      *p=0;
+      delete pp;
+      pp=0;
    }
    return res;
 }
@@ -566,6 +571,7 @@ void SFtp::SendRequest()
    switch((open_mode)mode)
    {
    case CHANGE_DIR:
+      LogNote(9,"checking directory `%s'",file.get());
       SendRequest(new Request_STAT(lc_to_utf8(file),0,protocol_version),Expect::CWD);
       SendRequest(new Request_STAT(lc_to_utf8(dir_file(file,".")),0,protocol_version),Expect::CWD);
       state=WAITING;
@@ -728,6 +734,13 @@ int SFtp::HandlePty()
 void SFtp::HandleExpect(Expect *e)
 {
    const Packet *reply=e->reply;
+   if(reply->TypeIs(SSH_FXP_STATUS))
+   {
+      Reply_STATUS *r=(Reply_STATUS*)reply;
+      const char *message=r->GetMessage();
+      LogNote(9,"status code=%d(%s), message=%s",r->GetCode(),r->GetCodeText(),
+	 message?message:"NULL");
+   }
    switch(e->tag)
    {
    case Expect::FXP_VERSION:
@@ -918,29 +931,29 @@ void SFtp::HandleExpect(Expect *e)
       }
       break;
    case Expect::INFO:
+      entity_size=NO_SIZE;
+      entity_date=NO_DATE;
       if(reply->TypeIs(SSH_FXP_ATTRS))
       {
 	 const FileAttrs *a=((Reply_ATTRS*)reply)->GetAttrs();
-	 entity_size=NO_SIZE;
-	 entity_date=NO_DATE;
 	 if(a->flags&SSH_FILEXFER_ATTR_SIZE)
 	    entity_size=a->size;
 	 if(a->flags&SSH_FILEXFER_ATTR_MODIFYTIME)
 	    entity_date=a->mtime;
-	 if(mode==ARRAY_INFO)
-	 {
-	    array_for_info[e->i].size=entity_size;
-	    array_for_info[e->i].get_size=false;
-	    array_for_info[e->i].time=entity_date;
-	    array_for_info[e->i].get_time=false;
-	    break;
-	 }
 	 LogNote(9,"file info: size=%lld, date=%s",(long long)entity_size,ctime(&entity_date));
-	 if(opt_size)
-	    *opt_size=entity_size;
-	 if(opt_date)
-	    *opt_date=entity_date;
       }
+      if(mode==ARRAY_INFO)
+      {
+	 array_for_info[e->i].size=entity_size;
+	 array_for_info[e->i].get_size=false;
+	 array_for_info[e->i].time=entity_date;
+	 array_for_info[e->i].get_time=false;
+	 break;
+      }
+      if(opt_size)
+	 *opt_size=entity_size;
+      if(opt_date)
+	 *opt_date=entity_date;
       break;
    case Expect::WRITE_STATUS:
       if(reply->TypeIs(SSH_FXP_STATUS))
@@ -1843,9 +1856,19 @@ SFtp::unpack_status_t SFtp::Reply_STATUS::Unpack(const Buffer *b)
    UNPACK32(code);
    if(protocol_version>=3)
    {
+      if(unpacked>=limit)
+      {
+	 LogError(2,"Status reply lacks `error message' field");
+	 return UNPACK_SUCCESS;
+      }
       res=Packet::UnpackString(b,offset,limit,&message);
       if(res!=UNPACK_SUCCESS)
 	 return res;
+      if(unpacked>=limit)
+      {
+	 LogError(2,"Status reply lacks `language tag' field");
+	 return UNPACK_SUCCESS;
+      }
       res=Packet::UnpackString(b,offset,limit,&language);
       if(res!=UNPACK_SUCCESS)
 	 return res;
@@ -1914,6 +1937,12 @@ const char *SFtp::lc_to_utf8(const char *s)
    return s;
 }
 
+FileSet *SFtp::GetFileSet()
+{
+   FileSet *fset=file_set.borrow();
+   return fset?fset:new FileSet;
+}
+
 FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
 {
    const FileAttrs *a=&na->attrs;
