@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.h,v 1.27 2008/02/26 15:46:57 lav Exp $ */
+/* $Id: SFtp.h,v 1.28 2008/05/16 08:45:01 lav Exp $ */
 
 #ifndef SFTP_H
 #define SFTP_H
@@ -259,7 +259,7 @@ public:
       static bool is_valid_reply(int p)
       {
 	 return p==SSH_FXP_VERSION
-	    || p>=101 && p<=105
+	    || (p>=101 && p<=105)
 	    || p==SSH_FXP_EXTENDED_REPLY;
       }
    protected:
@@ -768,7 +768,7 @@ public:
    void Cleanup();
    void CleanupThis();
 
-   FileSet *GetFileSet() { FileSet *fset=file_set.borrow(); return fset?fset:new FileSet; }
+   FileSet *GetFileSet();
 };
 
 class SFtpDirList : public DirList
