@@ -18,27 +18,29 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.h,v 1.7 2002/07/31 11:28:01 lav Exp $ */
+/* $Id: SleepJob.h,v 1.15 2008/08/05 12:10:12 lav Exp $ */
 
 #ifndef SLEEPJOB_H
 #define SLEEPJOB_H
 
 #include "Job.h"
 #include "ResMgr.h"
+#include "LocalDir.h"
+#include "CmdExec.h"
 
-class LocalDirectory;
-
-class SleepJob : public SessionJob
+class SleepJob : public SessionJob, public Timer
 {
-   time_t start_time;
-   TimeInterval next_time;
-   char *cmd;
+   xstring cmd;
    int exit_code;
    bool done;
-   LocalDirectory *saved_cwd;
-   class CmdExec *exec;
-   bool   repeat;
-   int    repeat_count;
+   Ref<LocalDirectory> saved_cwd;
+   JobRef<CmdExec> exec;
+   bool repeat;
+   bool weak;  // terminate on `exit bg'.
+   int repeat_count;
+   int max_repeat_count;
+   int continue_code;
+   int break_code;
 
 public:
    int Do();
@@ -50,7 +52,10 @@ public:
 
    void PrintStatus(int v,const char *);
 
-   void Repeat() { repeat=true; start_time-=next_time.Seconds(); }
+   void Repeat(int m) { repeat=true; max_repeat_count=m; Stop(); }
+   void SetWeak(bool w) { weak=w; }
+   void ContinueCode(int c) { continue_code=c; }
+   void BreakCode(int c) { break_code=c; }
 
    void lftpMovesToBackground();
 };
