@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SysCmdJob.cc,v 1.5 2007/08/02 11:27:59 lav Exp $ */
+/* $Id: SysCmdJob.cc,v 1.6 2008/08/05 12:10:12 lav Exp $ */
 
 #include <config.h>
 
@@ -32,22 +32,18 @@
 #include "misc.h"
 
 SysCmdJob::SysCmdJob(const char *c)
+   : cmd(c)
 {
-   w=0;
-   cmd=xstrdup(c);
 }
 
-SysCmdJob::~SysCmdJob()
+void SysCmdJob::PrepareToDie()
 {
    Bg();
    AcceptSig(SIGTERM);
-
    if(w)
-   {
-      w->Auto();
-      w=0;
-   }
+      w.borrow()->Auto();
 }
+SysCmdJob::~SysCmdJob() {}
 
 int SysCmdJob::Do()
 {
@@ -71,9 +67,9 @@ int SysCmdJob::Do()
       kill(getpid(),SIGSTOP);
       SignalHook::RestoreAll();
       if(cmd)
-	 execlp(shell,basename_ptr(shell),"-c",cmd,(char*)NULL);
+	 execlp(shell,basename_ptr(shell),"-c",cmd.get(),NULL);
       else
-	 execlp(shell,basename_ptr(shell),(char*)NULL);
+	 execlp(shell,basename_ptr(shell),NULL);
       fprintf(stderr,_("execlp(%s) failed: %s\n"),shell,strerror(errno));
       fflush(stderr);
       _exit(1);
