@@ -325,9 +325,11 @@ DataRecoder::DataRecoder(const char *from_code,const char *to_code,bool translit
 
 void DirectedBuffer::SetTranslation(const char *enc,bool translit)
 {
-   if(!enc)
+   if(!enc || !*enc)
       return;
    const char *local_code=ResMgr::Query("file:charset",0);
+   if(!local_code || !*local_code)
+      return;
    const char *from_code=(mode==GET?enc:local_code);
    const char *to_code  =(mode==GET?local_code:enc);
    SetTranslator(new DataRecoder(from_code,to_code,translit));
@@ -393,6 +395,10 @@ void IOBuffer::Put(const char *buf,int size)
    }
    DirectedBuffer::Put(buf,size);
 }
+void IOBuffer::Put(const char *buf)
+{
+   Put(buf,strlen(buf));
+}
 
 int IOBuffer::Do()
 {
@@ -639,6 +645,9 @@ bool IOBufferFDStream::Done()
    return false;
 }
 
+IOBufferFDStream::~IOBufferFDStream() {}
+
+
 // IOBufferFileAccess implementation
 #undef super
 #define super IOBuffer
@@ -728,7 +737,7 @@ void Buffer::PackINT64BE(long long data)
 {
    unsigned long long n;
    if(data<0)
-      n=(unsigned long long)(-data)^0xFFFFFFFFFFFFFFFFULL+1;
+      n=((unsigned long long)(-data)^0xFFFFFFFFFFFFFFFFULL)+1;
    else
       n=(unsigned long long)data;
    PackUINT64BE(n);
@@ -749,7 +758,7 @@ void Buffer::PackINT32BE(int data)
 {
    unsigned n;
    if(data<0)
-      n=(unsigned)(-data)^0xFFFFFFFFU+1;
+      n=((unsigned)(-data)^0xFFFFFFFFU)+1;
    else
       n=(unsigned)data;
    PackUINT32BE(n);
