@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.54 2007/09/10 11:45:28 lav Exp $ */
+/* $Id: buffer.h,v 1.56 2008/08/05 12:10:14 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
@@ -215,7 +215,7 @@ public:
 
    // Put method with Put_LL shortcut
    void Put(const char *,int);
-   void Put(const char *buf) { Put(buf,strlen(buf)); }
+   void Put(const char *buf);
    void Put(const xstring &s) { Put(s.get(),s.length()); }
    // anchor to PutEOF_LL
    void PutEOF() { DirectedBuffer::PutEOF(); PutEOF_LL(); }
@@ -237,7 +237,8 @@ public:
 
 class IOBufferFDStream : public IOBuffer
 {
-   Ref<FDStream> stream;
+   Ref<FDStream> my_stream;
+   const Ref<FDStream>& stream;
    Ref<Timer> put_ll_timer;
 
    int Get_LL(int size);
@@ -245,9 +246,14 @@ class IOBufferFDStream : public IOBuffer
 
 public:
    IOBufferFDStream(FDStream *o,dir_t m)
+      : IOBuffer(m), my_stream(o), stream(my_stream) {}
+   IOBufferFDStream(const Ref<FDStream>& o,dir_t m)
       : IOBuffer(m), stream(o) {}
    IOBufferFDStream(FDStream *o,dir_t m,Timer *t)
+      : IOBuffer(m), my_stream(o), stream(my_stream), put_ll_timer(t) {}
+   IOBufferFDStream(const Ref<FDStream>& o,dir_t m,Timer *t)
       : IOBuffer(m), stream(o), put_ll_timer(t) {}
+   ~IOBufferFDStream();
    bool Done();
    FgData *GetFgData(bool fg);
    const char *Status() { return stream->status; }
