@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.286 2008/02/06 07:11:23 lav Exp $ */
+/* $Id: commands.cc,v 1.288 2008/05/22 13:20:38 lav Exp $ */
 
 #include <config.h>
 
@@ -76,11 +76,7 @@
 #include "LocalDir.h"
 #include "ConnectionSlot.h"
 
-#include "confpaths.h"
-
-#define MINUTE (60)
-#define HOUR   (60*MINUTE)
-#define DAY    (24*HOUR)
+#include "configmake.h"
 
 History	 cwd_history;
 
