@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: complete.cc,v 1.72 2006/02/06 10:57:30 lav Exp $ */
+/* $Id: complete.cc,v 1.78 2008/05/22 13:20:38 lav Exp $ */
 
 #include <config.h>
 
@@ -27,7 +27,6 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
-#include <glob.h>
 #include <sys/time.h>
 #include <assert.h>
 #include "xmalloc.h"
@@ -47,6 +46,8 @@
 #include "misc.h"
 
 CDECL_BEGIN
+#include <glob.h>
+
 #define USE_VARARGS 1
 #define PREFER_STDARG 1
 #include "readline/readline.h"
@@ -56,7 +57,7 @@ CDECL_END
 # define GLOB_PERIOD 0
 #endif
 
-static char *bash_dequote_filename (char *text, int quote_char);
+static char *bash_dequote_filename (const char *text, int quote_char);
 static int lftp_char_is_quoted(const char *string,int eindex);
 
 static int len;    // lenght of the word to complete
@@ -82,7 +83,7 @@ char *command_generator(const char *text,int state)
    }
 
    /* Return the next name which partially matches from the command list. */
-   while ((name=CmdExec::CmdByIndex(cindex)->name)!=0)
+   while ((name=CmdExec::CmdByIndex(cindex))!=0)
    {
       cindex++;
       if(name[0]=='.' && len==0)
@@ -503,7 +504,7 @@ static completion_type cmd_completion_type(const char *cmd,int start)
 	 return LOCAL;
       if(have_N)
 	 arg--;
-      if(have_R ^ arg!=1)
+      if(have_R ^ (arg!=1))
 	 return LOCAL_DIR;
       if(arg>2)
 	 return NO_COMPLETION;
@@ -661,8 +662,7 @@ static char **lftp_completion (const char *text,int start,int end)
 	 goto really_remote;
       }
    really_local:
-      ArgV arg("", ResMgr::Query("cmd:cls-completion-default", 0));
-      fso.parse_argv(&arg);
+      fso.parse_res(ResMgr::Query("cmd:cls-completion-default", 0));
 
       bool tilde_expanded=false;
       const char *home=getenv("HOME");
@@ -764,16 +764,15 @@ static char **lftp_completion (const char *text,int start,int end)
 
       rl_save_prompt();
 
-      ArgV arg("", ResMgr::Query("cmd:cls-completion-default", 0));
-      fso.parse_argv(&arg);
+      fso.parse_res(ResMgr::Query("cmd:cls-completion-default", 0));
 
       if(rg)
       {
-	 rg->glob->NoMatchPeriod();
-	 rg->glob->NoInhibitTilde();
+	 rg->NoMatchPeriod();
+	 rg->NoInhibitTilde();
 	 if(fso.patterns_casefold) {
 	    rl_variable_bind("completion-ignore-case", "1");
-	    rg->glob->CaseFold();
+	    rg->CaseFold();
 	 } else
 	    rl_variable_bind("completion-ignore-case", "0");
 
@@ -838,8 +837,7 @@ static char **lftp_completion (const char *text,int start,int end)
    char quoted=((lftp_char_is_quoted(rl_line_buffer,start) &&
 		 strchr(rl_completer_quote_characters,rl_line_buffer[start-1]))
 		? rl_line_buffer[start-1] : 0);
-   char *textq=alloca_strdup(text);
-   textq=bash_dequote_filename(textq, quoted);
+   xstring_ca textq(bash_dequote_filename(text, quoted));
    len=strlen(textq);
 
    char **matches=lftp_rl_completion_matches(textq,generator);
@@ -855,14 +853,12 @@ static char **lftp_completion (const char *text,int start,int end)
    if(!matches)
    {
       rl_attempted_completion_over = 1;
-      goto leave;
+      return 0;
    }
 
    if(type==REMOTE_DIR)
       rl_completion_append_character='/';
 
-leave:
-   xfree(textq);
    return matches;
 }
 
@@ -1032,9 +1028,11 @@ contains_shell_metas (char *string)
    allows single quotes to appear within double quotes, and vice versa.
    It should be smarter. */
 static char *
-bash_dequote_filename (char *text, int quote_char)
+bash_dequote_filename (const char *text, int quote_char)
 {
-  char *ret, *p, *r;
+  char *ret;
+  const char *p;
+  char *r;
   int l, quoted;
 
   l = strlen (text);
@@ -1295,7 +1293,7 @@ void lftp_readline_init ()
 
 extern "C" void completion_display_list (char **matches, int len)
 {
-   OutputJob *b=new OutputJob((FDStream *) NULL, "completion");
+   JobRef<OutputJob> b(new OutputJob((FDStream *) NULL, "completion"));
 
    if(glob_res) {
       /* Our last completion action was of files, and we kept that
@@ -1313,8 +1311,7 @@ extern "C" void completion_display_list (char **matches, int len)
       FileSetOutput fso;
       fso.config(b);
 
-      ArgV arg("", ResMgr::Query("cmd:cls-completion-default", 0));
-      fso.parse_argv(&arg);
+      fso.parse_res(ResMgr::Query("cmd:cls-completion-default", 0));
 
       fso.print(tmp, b);
    } else {
@@ -1338,6 +1335,4 @@ extern "C" void completion_display_list (char **matches, int len)
 	 break;
       }
    }
-
-   SMTask::Delete(b);
 }
