@@ -18,21 +18,22 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: mgetJob.h,v 1.11 2007/04/13 12:18:33 lav Exp $ */
+/* $Id: mgetJob.h,v 1.12 2008/08/05 12:10:16 lav Exp $ */
 
 #ifndef MGETJOB_H
 #define MGETJOB_H
 
 #include "GetJob.h"
 #include "FileGlob.h"
+#include "mkdirJob.h"
 
 class mgetJob : public GetJob
 {
-   GlobURL *rg;
-   class mkdirJob *mkdir_job;
-   ArgV *mkdir_args;
+   Ref<GlobURL> rg;
+   JobRef<mkdirJob> mkdir_job;
+   Ref<ArgV> mkdir_args;
    int mkdir_base_arg;
-   ArgV *m_args;
+   Ref<ArgV> m_args;
    xstring_c output_dir;
 
    bool	 make_dirs;
