@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.93 2007/11/13 07:55:02 lav Exp $ */
+/* $Id: misc.cc,v 1.95 2008/07/23 08:46:12 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -42,7 +42,7 @@
 #endif
 
 CDECL_BEGIN
-#include <regex.h>
+#include "regex.h"
 CDECL_END
 #include "misc.h"
 #include "ProcWait.h"
@@ -50,10 +50,7 @@ CDECL_END
 #include "url.h"
 #include "ResMgr.h"
 #include <mbswidth.h>
-
-#if HAVE_UNSETENV && !HAVE_DECL_UNSETENV
-CDECL void unsetenv(const char *name);
-#endif
+#include "strftime.h"
 
 const char *dir_file(const char *dir,const char *file)
 {
@@ -487,30 +484,6 @@ mktime_from_utc (const struct tm *t)
    return (tl <= tb ? (tl + (tl - tb)) : (tl - (tb - tl)));
 }
 
-#ifndef HAVE_UNSETENV
-# ifdef HAVE_ENVIRON
-extern char **environ;
-void unsetenv(const char *env)
-{
-   char **scan=environ;
-   if(!scan)
-      return;
-   int env_len=strlen(env);
-   while(*scan)
-   {
-      if(!strncmp(*scan,env,env_len) && (*scan)[env_len]=='=')
-	 break;
-      scan++;
-   }
-   while(*scan)
-   {
-      *scan=*(scan+1);
-      scan++;
-   }
-}
-# endif
-#endif
-
 static void set_tz(const char *tz)
 {
    static char *put_tz;
