@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,9 +28,10 @@
 
 class mkdirJob : public SessionJob
 {
-   ArgV	 *args;
+   Ref<ArgV> args;
    const char *curr,*first;
-   FA	 *orig_session;
+   FileAccessRef my_session;
+   FileAccessRefC session;
    int	 failed,file_count;
    bool	 quiet;
    bool	 opt_p;
@@ -41,11 +42,10 @@ public:
    int	 ExitCode() { return failed; }
 
    void	 PrintStatus(int,const char *);
-   void	 ShowRunStatus(StatusLine *);
+   void	 ShowRunStatus(const SMTaskRef<StatusLine>&);
    void	 SayFinal();
 
    mkdirJob(FileAccess *session,ArgV *a);
-   ~mkdirJob();
 
    void	 BeQuiet() { quiet=true; }
 
